Bridge = Core.class(Sprite)

function Bridge:init(xworld, xposx, xposy)
	self.bridgeactors = {}
	-- create ground body
	local bridgeground = xworld:createBody({})
	bridgeground:setPosition(xposx, xposy)
	-- this box shape will be used while creating the bridge elements
	local shape = b2.PolygonShape.new()
	local x = 22
	shape:setAsBox(x, 4)
	-- and our fixture definition
	local fixtureDef = {shape = shape, density = 1, friction = 0.2}
	-- start to create the bridge
	local prevBody = bridgeground
	for i = 0, 18 do
		local bodyDef = {type = b2.DYNAMIC_BODY, position = {x = xposx + i * x, y = xposy}}
		local body = xworld:createBody(bodyDef)
		body:createFixture(fixtureDef)
		local bitmap = Bitmap.new(Texture.new("gfx/bridge/rect.png", true))
		bitmap:setAnchorPoint(0.5, 0.5)
		bitmap:setScale(1.2)
		self:addChild(bitmap)
		self.bridgeactors[body] = bitmap
		-- attach each pair of bridge elements with revolute joint
		local jointDef = b2.createRevoluteJointDef(prevBody, body, xposx + i * x, xposy)
		xworld:createJoint(jointDef)
		prevBody = body
	end
	-- attach last bridge element to the ground body
	local jointDef = b2.createRevoluteJointDef(prevBody, bridgeground, xposx + 19 * x, xposy)
	xworld:createJoint(jointDef)
	
	self:addEventListener(Event.ENTER_FRAME, self.onEnterFrame, self)
end

function Bridge:onEnterFrame(e)
	for body, sprite in pairs(self.bridgeactors) do
		sprite:setPosition(body:getPosition())
		sprite:setRotation(body:getAngle() * 180 / math.pi)
	end
end
