Collectibles = Core.class(Sprite)

function Collectibles:init(xworld, xtexture, xscalex, xscaley, xfixedrotation,
			xdensity, xrestitution, xfriction,
			xlvx, xlvy, xstartx, xstarty,
			xminposx, xminposy, xmaxposx, xmaxposy,
			xtype)
	self.world = xworld

	local mytexture = Texture.new(xtexture, true, {wrap = Texture.REPEAT})
	self.bitmap = Bitmap.new(mytexture)
	self.bitmap:setAnchorPoint(0.5, 0.5)
	self.bitmap:setScale(xscalex, xscaley)
	self:addChild(self.bitmap)

	self.body = xworld:createBody { type = b2.KINEMATIC_BODY }
	self.body.type = xtype
	self.body.iscollected = false
	self.body:setPosition(xstartx, xstarty)
	self.body:setFixedRotation(xfixedrotation)
	local shape = b2.PolygonShape.new()
	shape:setAsBox(self.bitmap:getWidth() / 2, self.bitmap:getHeight() / 2)
	local fixture = self.body:createFixture {
		shape = shape, 
		density = xdensity, restitution = xrestitution, friction = xfriction,
		isSensor = true
	}
	local filterData = { categoryBits = g_collectible, maskBits = g_player, groupIndex = 0 }
	fixture:setFilterData(filterData)
	fixture = nil
	-- moving or not moving?
	self.lvx, self.lvy = xlvx, xlvy
	self.startx, self.starty = xstartx, xstarty
	self.minposx, self.minposy = xminposx, xminposy
	self.maxposx, self.maxposy = xmaxposx, xmaxposy
	self.distx, self.disty = xminposx or 0, xminposy or 0
	self.body:setLinearVelocity(xlvx or 0, xlvy or 0)

	-- listeners
	self:addEventListener(Event.ENTER_FRAME, self.onEnterFrame, self)
	self.world:addEventListener(Event.BEGIN_CONTACT, self.onBeginContact, self)
	self.world:addEventListener(Event.END_CONTACT, self.onEndContact, self)
	self.world:addEventListener(Event.PRE_SOLVE, self.onPreSolveContact, self)
	self.world:addEventListener(Event.POST_SOLVE, self.onPostSolveContact, self)
end

-- game loop
function Collectibles:onEnterFrame(e)
	if self.body ~= nil then
		if self.body.iscollected then
			self.world:destroyBody(self.body)
			self:getParent():removeChild(self)
			self.body = nil
			g_score += 5
			return
		end
		-- bitmap
		self.bitmap:setPosition(self.body:getPosition())
		self.bitmap:setRotation(self.body:getAngle() * 180 / math.pi)
	end
end

-- collisions handler
function Collectibles:onBeginContact(e)
	local bodyA = e.fixtureA:getBody()
	local bodyB = e.fixtureB:getBody()
	if bodyA.type == 50 and bodyB.type == 2 then
		bodyA.iscollected = true
	end
	if bodyA.type == 2 and bodyB.type == 50 then
		bodyB.iscollected = true
	end
end

function Collectibles:onPreSolveContact(e)
	local bodyA = e.fixtureA:getBody()
	local bodyB = e.fixtureB:getBody()
	if bodyA.type == 50 and bodyB.type == 100 then
		e.contact:setEnabled(false)
	end
	if bodyA.type == 100 and bodyB.type == 50 then
		e.contact:setEnabled(false)
	end
end

function Collectibles:onPostSolveContact(e)
end

function Collectibles:onEndContact(e)
end
