LF_Bullet = Core.class(Sprite)

function LF_Bullet:init(xworld, xtexture, xscalex, xscaley, xfixedrotation,
			xdensity, xrestitution, xfriction,
			xlvx, xlvy, xstartx, xstarty,
			xfixturetype)
	self.world = xworld
	self.maxx = xstartx + 12 * 32
	self.removebullet = false

	local texture = Texture.new(xtexture, true, {wrap = Texture.REPEAT})
	self.bitmap = Bitmap.new(texture)
	self.bitmap:setAnchorPoint(0.5, 0.5)
	self.bitmap:setScale(xscalex, xscaley)
	self:addChild(self.bitmap)

	self.body = xworld:createBody { type = b2.DYNAMIC_BODY }
	self.body.type = xfixturetype
--	self.body:setBullet(true)
	self.body:setGravityScale(0)
	self.body:setPosition(xstartx, xstarty)
	self.body:setFixedRotation(xfixedrotation)
	self.shape = b2.PolygonShape.new()
	self.shape:setAsBox(self.bitmap:getWidth() / 2, self.bitmap:getHeight() / 2)
	local fixture = self.body:createFixture {
		shape = self.shape, 
		density = xdensity, restitution = xrestitution, friction = xfriction
	}
	local filterData = { categoryBits = g_bullet, maskBits = g_wall + g_nme + g_platform + g_passthroughplatform, groupIndex = 0 }
	fixture:setFilterData(filterData)
	fixture = nil
 	self.body:setLinearVelocity(xlvx or 0, xlvy or 0)

	-- listeners
	self:addEventListener(Event.ENTER_FRAME, self.onEnterFrame, self)
	self.world:addEventListener(Event.BEGIN_CONTACT, self.onBeginContact, self)
	self.world:addEventListener(Event.END_CONTACT, self.onEndContact, self)
	self.world:addEventListener(Event.PRE_SOLVE, self.onPreSolveContact, self)
	self.world:addEventListener(Event.POST_SOLVE, self.onPostSolveContact, self)
end

-- game loop
function LF_Bullet:onEnterFrame(e)
	if self.body ~= nil then
		local tposx, tposy = self.body:getPosition()
		-- remove bullet?
		if self.removebullet or math.abs(tposx) > self.maxx or math.abs(tposy) > self.maxx then
			self.world:destroyBody(self.body)
			self:getParent():removeChild(self)
			self.body = nil
		end
		-- bitmap
		self.bitmap:setPosition(tposx, tposy)
	end
end

-- collisions handler
function LF_Bullet:onBeginContact(e)
	local bodyA = e.fixtureA:getBody()
	local bodyB = e.fixtureB:getBody()
	if bodyA.type == 100 or bodyB.type == 100 then
		self.removebullet = true
	end
	if bodyA.type == 200 and bodyB.type == 100 then
		bodyA.isdead = true
	end
	if bodyA.type == 100 and bodyB.type == 200 then
		bodyB.isdead = true
	end
	if bodyA.type == 201 and bodyB.type == 100 then
		bodyA.isdead = true
	end
	if bodyA.type == 100 and bodyB.type == 201 then
		bodyB.isdead = true
	end
end

function LF_Bullet:onPreSolveContact(e)
end

function LF_Bullet:onPostSolveContact(e)
end

function LF_Bullet:onEndContact(e)
end
