LF_Kinematic_Box = Core.class(Sprite)

function LF_Kinematic_Box:init(xworld, xtexture, xscalex, xscaley, xfixedrotation,
			xdensity, xrestitution, xfriction,
			xlvx, xlvy, xstartx, xstarty,
			xminposx, xminposy, xmaxposx, xmaxposy,
			xtype)
	local mytexture = Texture.new(xtexture, true, {wrap = Texture.REPEAT})
	self.mybitmap = Bitmap.new(mytexture)
	self.mybitmap:setAnchorPoint(0.5, 0.5)
	self.mybitmap:setScale(xscalex, xscaley)
	local w, h = self.mybitmap:getWidth(), self.mybitmap:getHeight()
	self:addChild(self.mybitmap)

	self.body = xworld:createBody { type = b2.KINEMATIC_BODY }
	self.body.type = xtype
	self.body:setPosition(xstartx, xstarty)
	self.body:setFixedRotation(xfixedrotation)
	local shape = b2.PolygonShape.new()
	shape:setAsBox(w / 2, h / 2)
	local fixture = self.body:createFixture {
		shape = shape,
		density = xdensity, restitution = xrestitution, friction = xfriction
	}
	local filterData = { categoryBits = g_platform, maskBits = g_nme + g_player, groupIndex = 0 }
	fixture:setFilterData(filterData)
	fixture = nil
	-- moving or not moving?
	self.lvx, self.lvy = xlvx, xlvy
	self.startx, self.starty = xstartx, xstarty
	self.minposx, self.minposy = xminposx, xminposy
	self.maxposx, self.maxposy = xmaxposx, xmaxposy
	self.distx, self.disty = xminposx or 0, xminposy or 0
	self.body:setLinearVelocity(xlvx or 0, xlvy or 0)

	-- listeners
	self:addEventListener(Event.ENTER_FRAME, self.onEnterFrame, self)
end

function LF_Kinematic_Box:onEnterFrame(dt)
	-- body
	local pfposx, pfposy = self.body:getPosition()
	if pfposx <= self.minposx then
		if pfposy <= self.minposy then
			self.body:setLinearVelocity(self.lvx, self.lvy)
		elseif pfposy >= self.maxposy then
			self.body:setLinearVelocity(self.lvx, -self.lvy)
		end
	elseif pfposx >= self.maxposx then
		if pfposy <= self.minposy then
			self.body:setLinearVelocity(-self.lvx, self.lvy)
		elseif pfposy >= self.maxposy then
			self.body:setLinearVelocity(-self.lvx, -self.lvy)
		end
	end
	-- bitmap
	self.mybitmap:setPosition(pfposx, pfposy)
end
