Player = Core.class(Sprite)

function Player:init(xworld, xspritesheet, xcols, xrows)
	-- settings
	self.world = xworld
	self.lives = 3
	self.nrg = 10
	self.washit = false
	self.speed = 5
	self.flip = 1
	self.numfeetcontacts = 0 -- is on ground
	self.numplatformcontacts = 0 -- is on platform
	self.numenemycontacts = 0 -- collide with enemy
	self.isleft, self.isright, self.isdown = false, false, false -- controls
	self.isgoingdownplatform = false

	-- animations
	self.currentanim = ""
	self.frame = 0
	self.animspeed = 1 / 6
	self.animtimer = self.animspeed
	-- retrieve all anims in spritesheet
	local myanimstex = Texture.new(xspritesheet)
	local w, h = myanimstex:getWidth() / xcols, myanimstex:getHeight() / xrows
	local myanims_list = {}
	for r = 1, xrows do
		for c = 1, xcols do
			local myanimstexregion = TextureRegion.new(myanimstex, (c - 1) * w, (r - 1) * h, w, h)
			myanims_list[#myanims_list + 1] = myanimstexregion
		end
	end
	-- create anims 1
	self.anims = {}
	function createAnim(xanimname, xstart, xfinish, xanimslist)
		self.anims[xanimname] = {}
		for i = xstart, xfinish do
			self.anims[xanimname][#self.anims[xanimname] + 1] = xanimslist[i]
		end
	end
	-- create anims 2
	createAnim("idle", 1, 2, myanims_list)
	createAnim("jump_up", 38, 38, myanims_list)
	createAnim("jump_down", 40, 40, myanims_list)
	createAnim("walk", 13, 16, myanims_list)

	-- the sprite & the bitmap
	local mysprite = Sprite.new()
	self.bitmap = Bitmap.new(myanims_list[1]) -- starting bmp texture
	self.bitmap:setAnchorPoint(0.5, 0.5)
	self.bitmap:setPosition(w / 2, h / 2)
	mysprite:addChild(self.bitmap)
	self:addChild(mysprite)

	-- the dynamic body (liquidfun)
	self.body = xworld:createBody { type = b2.DYNAMIC_BODY }
	self.body:setFixedRotation(true)
	self.body.type = 2
--	local bodyshape = b2.PolygonShape.new()
--	bodyshape:setAsBox(w / 4 - 12, h / 2, w / 2, h / 2, 0)
	local bodyshape = b2.CircleShape.new(w / 2, h - 14, 14)
	local fixture = self.body:createFixture {
		shape = bodyshape, density = 4, restitution = 0, friction = 1
	}
	local filterData = { categoryBits = g_player, maskBits = g_wall + g_nme + g_collectible + g_platform + g_passthroughplatform, groupIndex = 0 }
	fixture:setFilterData(filterData)
	fixture = nil
	-- the left and right body with no friction so does not stick on walls!
	local myleftshape = b2.PolygonShape.new()
	myleftshape:setAsBox(2, h / 2 - 6, w / 4 + 2, h / 2 - 6 / 2, 0)
	local myleftfixture = self.body:createFixture { shape = myleftshape, friction = 0 }
	myleftfixture = nil
	local myrightshape = b2.PolygonShape.new()
	myrightshape:setAsBox(2, h / 2 - 6, w - w / 4 - 2, h / 2 - 6 / 2, 0)
	local myrightfixture = self.body:createFixture { shape = myrightshape, friction = 0 }
	myrightfixture = nil
	-- the feet sensor / is on ground
	local myfeetshape = b2.PolygonShape.new()
	myfeetshape:setAsBox(w / 4, 8, w / 2, h, 0)
	local myfeetfixture = self.body:createFixture { shape = myfeetshape, isSensor = true }
	myfeetfixture = nil

	-- listeners
	self:addEventListener(Event.ENTER_FRAME, self.onEnterFrame, self)
	self:addEventListener(Event.KEY_DOWN, self.onKeyDown, self)
	self:addEventListener(Event.KEY_UP, self.onKeyUp, self)
	self.world:addEventListener(Event.BEGIN_CONTACT, self.onBeginContact, self)
	self.world:addEventListener(Event.END_CONTACT, self.onEndContact, self)
	self.world:addEventListener(Event.PRE_SOLVE, self.onPreSolveContact, self)
	self.world:addEventListener(Event.POST_SOLVE, self.onPostSolveContact, self)
end

-- update player
function Player:onEnterFrame(e)
	local posx, posy = self.body:getPosition()
	local vx, vy = self.body:getLinearVelocity()
	-- anim state
	if self.numenemycontacts > 0 and self.washit then
		self.nrg -= 1
		self.washit = false
	end
	if self.nrg <= 0 then
		self.lives -= 1
		self.nrg = 10
	end
	if self.lives <= 0 then
		scenemanager:changeScene("level01", 1, transitions[1], easing.inBack)
	end
	if self.numfeetcontacts > 0 then -- is on floor anims
		if vx == 0 then
			self.currentanim = "idle"
		elseif vx ~= 0 then
			if self.numplatformcontacts > 0 and not self.isleft and not self.isright then
				self.currentanim = "idle"
			else
				self.currentanim = "walk"
			end
		end
	else -- is not on floor anims
		if vy < 0 then
			self.currentanim = "jump_up"
		elseif vy >= 0 then
			self.currentanim = "jump_down"
		end
	end
	-- anim loop
	if self.currentanim ~= "" then
		self.animtimer = self.animtimer - e.deltaTime
		if self.animtimer <= 0 then
			self.frame += 1
			self.animtimer = self.animspeed
			if self.frame > #self.anims[self.currentanim] then
				self.frame = 1
			end
			self.bitmap:setTextureRegion(self.anims[self.currentanim][self.frame])
		end
	end
	-- player controls (gradual velocity)
	if self.isleft and not self.isright then
		vx = math.max(vx - 1, -self.speed)
		self.flip = -1
	elseif self.isright and not self.isleft then
		vx = math.min(vx + 1, self.speed)
		self.flip = 1
	end
	self.body:setLinearVelocity(vx, vy)
	-- move player sprite along with player physic body
	self:setPosition(posx, posy)
	self.bitmap:setScaleX(self.flip) -- left/right facing
end

-- collisions handler
function Player:onBeginContact(e)
	local bodyA = e.fixtureA:getBody()
	local bodyB = e.fixtureB:getBody()
	if bodyA.type == 2 or bodyB.type == 2 then
		-- is on floor
		self.numfeetcontacts += 1
	end
	-- nmes
	if bodyA.type == 2 and bodyB.type == 200 or bodyA.type == 200 and bodyB.type == 2 then
		self.numenemycontacts += 1
		self.washit = true
	end
	if bodyA.type == 2 and bodyB.type == 201 or bodyA.type == 201 and bodyB.type == 2 then
		self.numenemycontacts += 1
		self.washit = true
	end
	-- platforms
	if bodyA.type == 10 or bodyB.type == 10 then
		-- is on pass through platform
		self.numplatformcontacts += 1
	end
	if bodyA.type == 11 or bodyB.type == 11 then
		-- is on solid platform
		self.numplatformcontacts += 1
	end
end

function Player:onPreSolveContact(e)
	local bodyA = e.fixtureA:getBody()
	local bodyB = e.fixtureB:getBody()
	local platform = nil
	if bodyA.type == 10 then
		platform = bodyA
	end
	if bodyB.type == 10 then
		platform = bodyB
	end
	if platform == nil then return end

	-- going down platform
	if self.isgoingdownplatform then
		e.contact:setEnabled(false)
		return
	end
	-- pass through platform
	local velocityx, velocityy = self.body:getLinearVelocity()
	local posx, posy = self.body:getPosition()
	local pospx, pospy = platform:getPosition()
	if velocityy > 0 then -- going down = collision
		if self.isdown then -- player pressing down
			e.contact:setEnabled(false) -- else going up = no collision
		else
			e.contact:setEnabled(true)
		end
	elseif velocityy < -3 then -- going up = no collision (treshold) -7
		if math.abs(posy - pospy) < 80 then -- magic number
			e.contact:setEnabled(false)
		else
			e.contact:setEnabled(true)
		end
	end
end

function Player:onPostSolveContact(e)
end

function Player:onEndContact(e)
	local bodyA = e.fixtureA:getBody()
	local bodyB = e.fixtureB:getBody()
	if bodyA.type == 2 or bodyB.type == 2 then
		-- is not on floor
		self.numfeetcontacts -= 1
	end
	-- nmes
	if bodyA.type == 2 and bodyB.type == 200 or bodyA.type == 200 and bodyB.type == 2 then
		self.numenemycontacts -= 1
	end
	if bodyA.type == 2 and bodyB.type == 201 or bodyA.type == 201 and bodyB.type == 2 then
		self.numenemycontacts -= 1
	end
end

-- functions
function Player:jump()
	if self.numfeetcontacts > 0 then -- is on ground
		local gx, gy = self.world:getGravity()
		self.body:setLinearVelocity(0, -gy / 1.9)
		self.isgoingdownplatform = false
	end
end

function Player:shoot()
	local px, py = self.body:getPosition()
	local pmx = px + self:getWidth() / 2 -- get middle of sprite position
	self.bullet = LF_Bullet.new(self.world, "gfx/bg/mytile32.png", 1, 0.5, true,
			0.1, 1, 0,
			24 * self.flip, 0, pmx + 38 * self.flip, py + self:getHeight() / 2,
			100)
	self:getParent():addChild(self.bullet)
end

-- keys listeners
function Player:onKeyDown(e)
	if e.keyCode == KeyCode.LEFT then self.isleft = true end
	if e.keyCode == KeyCode.RIGHT then self.isright = true end
	if e.keyCode == KeyCode.UP then self:jump() end -- ***
	if e.keyCode == KeyCode.DOWN then
		self.isdown = true
		self.isgoingdownplatform = true
		self.body:setAwake(true)
	end
	if e.keyCode == KeyCode.SPACE then self:shoot() end -- ***
end

function Player:onKeyUp(e)
	if e.keyCode == KeyCode.LEFT then self.isleft = false end
	if e.keyCode == KeyCode.RIGHT then self.isright = false end
	if e.keyCode == KeyCode.DOWN then self.isdown = false end
end
